@@ -82,12 +82,12 @@ stop_bfd(int status)
 		return;
 
 	/* Stop daemon */
-	pidfile_rm(bfd_pidfile);
+	pidfile_rm(&bfd_pidfile);
 
 	/* Clean data */
-	free_global_data(global_data);
+	free_global_data(&global_data);
 	bfd_dispatcher_release(bfd_data);
-	free_bfd_data(bfd_data);
+	free_bfd_data(&bfd_data);
 	free_bfd_buffer();
 	thread_destroy_master(master);
 	free_parent_mallocs_exit();
@@ -159,8 +159,9 @@ start_bfd(__attribute__((unused)) data_t *prev_global_data)
 		init_global_data(global_data, prev_global_data, true);
 
 	/* Update process name if necessary */
-	if ((!reload && global_data->bfd_process_name) ||
-	    (reload &&
+	if ((!prev_global_data &&		// startup
+	    global_data->bfd_process_name) ||
+	    (prev_global_data &&		// reload
 	     (!global_data->bfd_process_name != !prev_global_data->bfd_process_name ||
 	      (global_data->bfd_process_name && strcmp(global_data->bfd_process_name, prev_global_data->bfd_process_name)))))
 		set_process_name(global_data->bfd_process_name);
@@ -240,7 +241,10 @@ static void
 bfd_signal_init(void)
 {
 	signal_set(SIGHUP, sigreload_bfd, NULL);
-	signal_set(SIGINT, sigend_bfd, NULL);
+	if (ignore_sigint)
+		signal_ignore(SIGINT);
+	else
+		signal_set(SIGINT, sigend_bfd, NULL);
 	signal_set(SIGTERM, sigend_bfd, NULL);
 	signal_set(SIGUSR1, sigdump_bfd, NULL);
 #ifdef THREAD_DUMP
@@ -284,8 +288,8 @@ reload_bfd_thread(__attribute__((unused)) thread_ref_t thread)
 	signal_set(SIGCHLD, thread_child_handler, master);
 	start_bfd(old_global_data);
 
-	free_bfd_data(old_bfd_data);
-	free_global_data(old_global_data);
+	free_bfd_data(&old_bfd_data);
+	free_global_data(&old_global_data);
 
 #ifndef _ONE_PROCESS_DEBUG_
 	save_config(true, "bfd", dump_bfd_data_global);
@@ -404,6 +408,8 @@ start_bfd_child(void)
 
 	prog_type = PROG_TYPE_BFD;
 
+	close_other_pidfiles();
+
 	/* Close the read end of the event notification pipes, and the track_process fd */
 #ifdef _WITH_VRRP_
 	close(bfd_vrrp_event_pipe[0]);
@@ -453,7 +459,7 @@ start_bfd_child(void)
 	separate_config_file();
 
 	/* Child process part, write pidfile */
-	if (!pidfile_write(bfd_pidfile, getpid())) {
+	if (!pidfile_write(&bfd_pidfile)) {
 		/* Fatal error */
 		log_message(LOG_INFO,
 			    "BFD child process: cannot write pidfile");
