@@ -17,22 +17,29 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
-#include <syslog.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <stdbool.h>
+
 #include "daemon.h"
 #include "logger.h"
 #include "utils.h"
 
 /* Daemonization function coming from zebra source code */
 pid_t
-xdaemon(int nochdir, int noclose, int exitflag)
+xdaemon(void)
 {
 	pid_t pid;
-	int ret;
+
+#ifdef ENABLE_LOG_TO_FILE
+	if (log_file_name)
+		flush_log_file();
+#endif
 
 	/* In case of fork is error. */
 	pid = fork();
@@ -42,32 +49,21 @@ xdaemon(int nochdir, int noclose, int exitflag)
 	}
 
 	/* In case of this is parent process. */
-	if (pid != 0) {
-		if (!exitflag)
-			exit(0);
-		else
-			return pid;
-	}
+	if (pid != 0)
+		return pid;
 
 	/* Become session leader and get pid. */
-	pid = setsid();
-	if (pid < -1) {
+	if (setsid() < 0) {
 		log_message(LOG_INFO, "xdaemon: setsid error");
 		return -1;
 	}
 
 	/* Change directory to root. */
-	if (!nochdir) {
-		ret = chdir("/");
-		if (ret < 0) {
-			log_message(LOG_INFO, "xdaemon: chdir error");
-		}
-	}
+	if (chdir("/") < 0)
+		log_message(LOG_INFO, "xdaemon: chdir error");
 
 	/* File descriptor close. */
-	if (!noclose)
-		set_std_fd(true);
+	set_std_fd(true);
 
-	umask(0);
 	return 0;
 }
