@@ -922,6 +922,14 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 	if (tb[IFA_ADDRESS] == NULL)
 		tb[IFA_ADDRESS] = tb[IFA_LOCAL];
 
+	/* Ignore tentative addresses - we can't do anything with them */
+#if HAVE_DECL_IFA_FLAGS
+	if ((tb[IFA_FLAGS] ? *(uint32_t *)RTA_DATA(tb[IFA_FLAGS]) : ifa->ifa_flags) & IFA_F_TENTATIVE)
+#else
+	if (ifa->ifa_flags & IFA_F_TENTATIVE)
+#endif
+		return 0;
+
 	/* local interface address */
 	addr.addr = (tb[IFA_LOCAL] ? RTA_DATA(tb[IFA_LOCAL]) : NULL);
 
@@ -1005,50 +1013,39 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 						else
 							is_tracking_saddr = false;
 
-						if (ifp == (
-#ifdef _HAVE_VRRP_VMAC_
-							    vrrp->family == AF_INET ? VRRP_CONFIGURED_IFP(vrrp) :
-#endif
-							    vrrp->ifp) &&
-						    vrrp->num_script_if_fault &&
-						    vrrp->family == ifa->ifa_family &&
+						if (vrrp->family == ifa->ifa_family &&
 						    vrrp->saddr.ss_family == AF_UNSPEC &&
-						    (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr)) {
-							/* Copy the address */
-							if (ifa->ifa_family == AF_INET)
-								inet_ip4tosockaddr(addr.in, &vrrp->saddr);
-							else
-								inet_ip6tosockaddr(addr.in6, &vrrp->saddr);
-							try_up_instance(vrrp, false);
-						}
+						    __test_bit(VRRP_FAULT_FL_NO_SOURCE_IP, &vrrp->flags_if_fault) &&
+						    vrrp->ifp) {
+							if (ifp == (
 #ifdef _HAVE_VRRP_VMAC_
-						/* If IPv6 link local and vmac doesn't have an address or we have
-						 * created one ourselves, add/replace the new address on the vmac */
-						else if (vrrp->family == AF_INET6 &&
-							 vrrp->ifp &&
-							 ifp == vrrp->ifp->base_ifp &&
-							 IS_MAC_IP_VLAN(vrrp->ifp) &&
-							 !__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->flags) &&
-							 ifa->ifa_family == AF_INET6 &&
-							 vrrp->ifp->is_ours) {
-								inet_ip6tosockaddr(addr.in6, &vrrp->saddr);
-#if 0
-								if (IN6_IS_ADDR_UNSPECIFIED(&vrrp->ifp->sin6_addr)) {
-									/* This should never happen with the current code since we always
-									 * create a link local address on the VMAC interface.
-									 * However, if in future it is decided not to automatically create
-									 * a link local address on the VMAC interface if the parent interface
-									 * does not have one, then we will need the following code
-									 */
-									if (add_link_local_address(vrrp->ifp, addr.in6) &&
-									    vrrp->num_script_if_fault &&
-									    (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr))
-										try_up_instance(vrrp, false);
-								} else
+								    vrrp->family == AF_INET ? VRRP_CONFIGURED_IFP(vrrp) :
+#endif
+								    vrrp->ifp) &&
+							    (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr)) {
+								/* Copy the address */
+								if (ifa->ifa_family == AF_INET)
+									inet_ip4tosockaddr(addr.in, &vrrp->saddr);
+								else
+									inet_ip6tosockaddr(addr.in6, &vrrp->saddr);
+								try_up_instance(vrrp, false, VRRP_FAULT_FL_NO_SOURCE_IP);
+							}
+#ifdef _HAVE_VRRP_VMAC_
+							/* If IPv6 link local and vmac doesn't have an address or we have
+							 * created one ourselves, add/replace the new address on the vmac */
+							else if (vrrp->family == AF_INET6 &&
+								 IS_MAC_IP_VLAN(vrrp->ifp) &&
+								 vrrp->ifp->is_ours &&
+								 ifp == vrrp->ifp->base_ifp &&
+								 !__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->flags)) {
+									inet_ip6tosockaddr(addr.in6, &vrrp->saddr);
+									if (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr)
+										try_up_instance(vrrp, false, VRRP_FAULT_FL_NO_SOURCE_IP);
+									else
+										reset_link_local_address(&vrrp->ifp->sin6_addr, vrrp);
+							}
 #endif
-									reset_link_local_address(&vrrp->ifp->sin6_addr, vrrp);
 						}
-#endif
 					}
 				}
 			}
@@ -1167,7 +1164,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 						}
 						else if (IF_ISUP(ifp)) {
 							/* We failed to add an address, so down the instance */
-							down_instance(vrrp);
+							down_instance(vrrp, VRRP_FAULT_FL_NO_SOURCE_IP);
 							vrrp->saddr.ss_family = AF_UNSPEC;
 						}
 					}
@@ -1187,7 +1184,10 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 						  IF_ISUP(ifp)) &&
 #endif
 						 (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr)) {
-						/* Don't attempt to send an IPv6 advert if no address on the interface */
+						/* Don't attempt to send an IPv6 advert if no address on the interface, but we can
+						 * for IPv4. So we want to clear the saddr before calling down_instance for IPv6,
+						 * but after for IPv4.
+						 */
 						if (vrrp->saddr.ss_family == AF_INET6
 #ifdef _HAVE_VRRP_VMAC_
 						    && !__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->flags)
@@ -1195,7 +1195,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 											)
 							vrrp->saddr.ss_family = AF_UNSPEC;
 
-						down_instance(vrrp);
+						down_instance(vrrp, VRRP_FAULT_FL_NO_SOURCE_IP);
 						vrrp->saddr.ss_family = AF_UNSPEC;
 					}
 				}
@@ -1601,10 +1601,26 @@ process_if_status_change(interface_t *ifp)
 		}
 
 		/* This vrrp's interface or underlying interface has changed */
-		if (now_up == (top->weight_multiplier == 1))
-			try_up_instance(vrrp, false);
-		else
-			down_instance(vrrp);
+		if (now_up == (top->weight_multiplier == 1)) {
+#ifdef _HAVE_VRRP_VMAC_
+			if (__test_bit(VRRP_VMAC_BIT, &vrrp->flags) &&
+					VRRP_CONFIGURED_IFP(vrrp) == ifp)
+				try_up_instance(vrrp, false, VRRP_FAULT_FL_BASE_INTERFACE_DOWN);
+			else
+#endif
+			{
+				/* assuming there is only one tracked interface per vrrp : to be checked */
+				try_up_instance(vrrp, false, VRRP_FAULT_FL_INTERFACE_DOWN);
+			}
+		} else {
+#ifdef _HAVE_VRRP_VMAC_
+			if (__test_bit(VRRP_VMAC_BIT, &vrrp->flags) &&
+					VRRP_CONFIGURED_IFP(vrrp) == ifp)
+				down_instance(vrrp, VRRP_FAULT_FL_BASE_INTERFACE_DOWN);
+			else
+#endif
+				down_instance(vrrp, VRRP_FAULT_FL_INTERFACE_DOWN);
+		}
 	}
 }
 
@@ -2200,7 +2216,6 @@ netlink_link_filter(__attribute__((unused)) struct sockaddr_nl *snl, struct nlms
 			if (strcmp(ifp->ifname, name)) {
 				/* The name can change, so handle that here */
 				log_message(LOG_INFO, "Interface name has changed from %s to %s", ifp->ifname, name);
-
 #ifndef _ONE_PROCESS_DEBUG_
 				if (prog_type != PROG_TYPE_VRRP) {
 					ifp->ifi_flags = 0;
