@@ -158,14 +158,16 @@ create_pidfile(pidfile_t *pidf)
 	struct stat st, fd_st;
 	int error;
 	int ret;
+#if HAVE_DECL_F_OFD_SETLK == 1
 	struct flock fl = { .l_type = F_WRLCK, .l_whence = SEEK_SET, .l_start = 0, .l_len = 0 };
+#endif
 
 	for (;;) {
 		/* We want to create the file with permissions rw-r--r-- */
 		if (umask_val & (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))
 			umask(umask_val & ~(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH));
 
-		while ((pidf->fd = open(pidf->path, O_NOFOLLOW | O_CREAT | O_WRONLY | O_NONBLOCK, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1 && errno == EINTR);
+		while ((pidf->fd = open(pidf->path, O_NOFOLLOW | O_CREAT | O_WRONLY | O_NONBLOCK | O_CLOEXEC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1 && errno == EINTR);
 		error = errno;
 
 		/* Restore the default umask */
@@ -177,6 +179,7 @@ create_pidfile(pidfile_t *pidf)
 			return true;
 		}
 
+#if HAVE_DECL_F_OFD_SETLK == 1
 		fl.l_pid = 0;
 		while ((ret = fcntl(pidf->fd, F_OFD_SETLK, &fl)) && errno == EINTR);
 		if (ret) {
@@ -187,6 +190,7 @@ create_pidfile(pidfile_t *pidf)
 
 			break;
 		}
+#endif
 
 		/* Make sure the file has not been removed/moved */
 		if (stat(pidf->path, &st)) {
@@ -278,7 +282,9 @@ pidfile_write(pidfile_t *pidf)
 					close(pidf->fd);
 				}
 
-				while ((pidf->fd = open(pidf->path, O_NOFOLLOW | O_WRONLY | O_NONBLOCK)) == -1 && errno == EINTR);
+				/* There is a window where the pid file could be deleted between us closing it and re-opening it,
+				 * so allow for creating the pidfile again. */
+				while ((pidf->fd = open(pidf->path, O_NOFOLLOW | O_CREAT | O_WRONLY | O_NONBLOCK | O_CLOEXEC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1 && errno == EINTR);
 
 				if (pidf->fd == -1)
 					return false;
