@@ -3,8 +3,8 @@
  *      data structure and functionality definitions
  */
 
-#ifndef KEEPALIVED_IP_VS_H
-#define KEEPALIVED_IP_VS_H
+#ifndef _KEEPALIVED_IP_VS_H
+#define _KEEPALIVED_IP_VS_H
 
 #include "config.h"
 
@@ -12,11 +12,7 @@
 #include <net/if.h>	/* Force inclusion of net/if.h before linux/if.h */
 #include <sys/types.h>
 #include <netinet/in.h>
-#ifdef HAVE_LINUX_IP_VS_H
 #include <linux/ip_vs.h>
-#else
-#include <net/ip_vs.h>
-#endif
 /* Prior to Linux 4.2 have to include linux/in.h and linux/in6.h
  * or linux/netlink.h to include linux/netfilter.h */
 #include <linux/netfilter.h>	/* For nf_inet_addr */
@@ -45,17 +41,17 @@ struct ip_vs_stats64 {
 	__u64	outbps;		/* current out byte rate */
 };
 typedef struct ip_vs_stats64 ip_vs_stats_t;
+#define	ip_vs_stats	stats
 #else
 typedef struct ip_vs_stats_user ip_vs_stats_t;
+#define	ip_vs_stats	user.stats
 #endif
 
 struct ip_vs_service_app {
 	struct ip_vs_service_user user;
 	uint16_t		af;
 	union nf_inet_addr	nf_addr;
-#ifdef _HAVE_PE_NAME_
 	char			pe_name[IP_VS_PENAME_MAXLEN + 1];
-#endif
 };
 
 struct ip_vs_dest_app {
@@ -74,50 +70,45 @@ struct ip_vs_dest_app {
 
 struct ip_vs_service_entry_app {
 	struct ip_vs_service_entry user;
+#ifdef _WITH_LVS_64BIT_STATS_
 	ip_vs_stats_t		stats;
+#endif
 	uint16_t		af;
 	union nf_inet_addr	nf_addr;
-#ifdef _HAVE_PE_NAME_
 	char			pe_name[IP_VS_PENAME_MAXLEN + 1];
-#endif
-
 };
 
 struct ip_vs_dest_entry_app {
 	struct ip_vs_dest_entry user;
+#ifdef _WITH_LVS_64BIT_STATS_
 	ip_vs_stats_t		stats;
+#endif
 	uint16_t		af;
 	union nf_inet_addr	nf_addr;
-
 };
 
 struct ip_vs_get_dests_app {
-	uint16_t		af;
-	union nf_inet_addr	nf_addr;
+	uint16_t		af;		/* Needed if don't get IPVS_DEST_ATTR_ADDR_FAMILY */
 
-	struct {
-	/* which service: user fills in these */
-	__u16			protocol;
-	__be32			addr;		/* virtual address */
-	__be16			port;
-	__u32			fwmark;		/* firwall mark of service */
+	unsigned		num_entries;	/* Number of entries space allocated for */
 
-	/* number of real servers */
-	unsigned int		num_dests;
+	struct ip_vs_get_dests_entries_app {
+		/* number of real servers */
+		unsigned int		num_dests;
 
-	/* the real servers */
-	struct ip_vs_dest_entry_app	entrytable[];
+		/* the real servers */
+		struct ip_vs_dest_entry_app	entrytable[];
 	} user;
 };
 
 /* The argument to IP_VS_SO_GET_SERVICES */
 struct ip_vs_get_services_app {
 	struct {
-	/* number of virtual services */
-	unsigned int		num_services;
+		/* number of virtual services */
+		unsigned int		num_services;
 
-	/* service table */
-	struct ip_vs_service_entry_app entrytable[0];
+		/* service table */
+		struct ip_vs_service_entry_app entrytable[];
 	} user;
 };
 
@@ -126,27 +117,8 @@ struct ip_vs_get_services_app {
 	#error The code assumes that IP_VS_IFNAME_MAXLEN <= IFNAMSIZ
 #endif
 
-/* The argument to IP_VS_SO_GET_DAEMON */
-struct ip_vs_daemon_kern {
-	/* sync daemon state (master/backup) */
-	int			state;
-
-	/* multicast interface name */
-	char			mcast_ifn[IP_VS_IFNAME_MAXLEN];
-
-	/* SyncID we belong to */
-	int			syncid;
-};
-
 struct ip_vs_daemon_app {
-	/* sync daemon state (master/backup) */
-	int			state;
-
-	/* multicast interface name */
-	char			mcast_ifn[IP_VS_IFNAME_MAXLEN];
-
-	/* SyncID we belong to */
-	int			syncid;
+	struct ip_vs_daemon_user user;
 
 #ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
 	/* UDP Payload Size */
@@ -166,4 +138,4 @@ struct ip_vs_daemon_app {
 #endif
 };
 
-#endif	/* KEEPALIVED_IP_VS_H */
+#endif	/* _KEEPALIVED_IP_VS_H */
