@@ -45,14 +45,22 @@
 #define	PID_EXTENSION		".pid"
 #define	RELOAD_EXTENSION	".reload"
 
+typedef struct pidfile {
+	const char *	path;
+	bool		free_path;
+	int		fd;
+} pidfile_t;
+
 extern const char *pid_directory;
 
 /* Prototypes */
 extern void create_pid_dir(void);
 extern void remove_pid_dir(void);
 extern char *make_pidfile_name(const char *, const char *, const char *);
-extern bool pidfile_write(const char *, int);
-extern void pidfile_rm(const char *);
+extern void pidfile_close(pidfile_t *, bool);
+extern bool pidfile_write(pidfile_t *);
+extern void pidfile_rm(pidfile_t *);
+extern void close_other_pidfiles(void);
 extern bool keepalived_running(unsigned long);
 
 #endif
