@@ -77,7 +77,8 @@ typedef struct _ip_address {
 #ifdef _WITH_NFTABLES_
 	bool			nftable_rule_set;	/* TRUE if in nftables set */
 #endif
-	bool			garp_gna_pending;	/* Is a gratuitous ARP/NA message still to be sent */
+	unsigned		garp_gna_pending;	/* Number of GARPs/GNAs still to be sent */
+	list_head_t		garp_gna_list;
 	uint32_t		preferred_lft;		/* IPv6 preferred_lft (0 means address deprecated) */
 
 	/* linked list member */
@@ -111,10 +112,11 @@ extern void dump_ipaddress(FILE *, const ip_address_t *);
 extern void dump_ipaddress_list(FILE *, const list_head_t *);
 extern ip_address_t *parse_ipaddress(ip_address_t *, const char *, bool);
 extern ip_address_t *parse_route(const char *);
-extern void alloc_ipaddress(list_head_t *, const vector_t *, bool);
+extern ip_address_t *alloc_ipaddress(const vector_t *, bool);
 extern void get_diff_address(vrrp_t *, vrrp_t *, list_head_t *);
 extern void clear_address_list(list_head_t *, bool);
 extern void clear_diff_static_addresses(void);
 extern void reinstate_static_address(ip_address_t *);
+extern void set_addrproto(void);
 
 #endif
