@@ -154,8 +154,8 @@ enum ip_route {
 	IPROUTE_PREF,
 	IPROUTE_FASTOPEN_NO_COOKIE,
 	IPROUTE_TTL_PROPAGATE,
-	IPROUTE_ADD,
-	IPROUTE_APPEND
+	IPROUTE_ADD_ROUTE,
+	IPROUTE_APPEND_ROUTE
 };
 
 #define	IPROUTE_BIT_DSFIELD	(1<<IPROUTE_DSFIELD)
@@ -181,8 +181,8 @@ enum ip_route {
 #define	IPROUTE_BIT_PREF	(1<<IPROUTE_PREF)
 #define	IPROUTE_BIT_FASTOPEN_NO_COOKIE	(1<<IPROUTE_FASTOPEN_NO_COOKIE)
 #define	IPROUTE_BIT_TTL_PROPAGATE (1<<IPROUTE_TTL_PROPAGATE)
-#define	IPROUTE_BIT_ADD		(1<<IPROUTE_ADD)
-#define	IPROUTE_BIT_APPEND	(1<<IPROUTE_APPEND)
+#define	IPROUTE_BIT_ADD		(1<<IPROUTE_ADD_ROUTE)
+#define	IPROUTE_BIT_APPEND	(1<<IPROUTE_APPEND_ROUTE)
 
 typedef struct _ip_route {
 	ip_address_t		*dst;
