@@ -45,8 +45,8 @@ typedef struct _sock {
 #ifdef _HAVE_VRF_
 	const interface_t	*vrf_ifp;
 #endif
-	const sockaddr_t	*unicast_src;
-	const sockaddr_t	*mcast_daddr;
+	sockaddr_t		*unicast_src;
+	sockaddr_t		*mcast_daddr;
 	int			fd_in;
 	int			fd_out;
 	int			rx_buf_size;
