@@ -122,8 +122,8 @@
 /* Used for initialising track files */
 static enum {
 	TRACK_FILE_NO_INIT,
-	TRACK_FILE_CREATE,
 	TRACK_FILE_INIT,
+	TRACK_FILE_OVERWRITE,
 } track_file_init;
 static int track_file_init_value;
 static tracked_file_t *current_tf;
@@ -337,7 +337,7 @@ track_file_init_handler(const vector_t *strvec)
 	const char *word;
 	int value;
 
-	track_file_init = TRACK_FILE_CREATE;
+	track_file_init = TRACK_FILE_INIT;
 	track_file_init_value = 0;
 
 	for (i = 1; i < vector_size(strvec); i++) {
@@ -360,7 +360,7 @@ track_file_init_handler(const vector_t *strvec)
 			track_file_init_value = value;
 		}
 		else if (!strcmp(word, "overwrite"))
-			track_file_init = TRACK_FILE_INIT;
+			track_file_init = TRACK_FILE_OVERWRITE;
 		else
 			report_config_error(CONFIG_GENERAL_ERROR, "Unknown track file init option %s", word);
 	}
@@ -479,7 +479,7 @@ track_file_end_handler(void)
 
 	if (track_file_init != TRACK_FILE_NO_INIT) {
 		ret = stat(track_file->file_path, &statb);
-		if (!ret && track_file_init == TRACK_FILE_INIT) {
+		if (!ret && track_file_init == TRACK_FILE_OVERWRITE) {
 			if ((statb.st_mode & S_IFMT) != S_IFREG) {
 				/* It is not a regular file */
 				report_config_error(CONFIG_GENERAL_ERROR, "Cannot initialise track file %s"
@@ -494,7 +494,8 @@ track_file_end_handler(void)
 		}
 
 		/* Don't overwrite a file on reload */
-		if (!reload && !__test_bit(CONFIG_TEST_BIT, &debug)) {
+		if (!reload && !__test_bit(CONFIG_TEST_BIT, &debug) &&
+		    (ret || track_file_init == TRACK_FILE_OVERWRITE)) {	// the file doesn't exist or we want to overwrite it
 			/* Write the value to the file */
 			if ((tf = fopen_safe(track_file->file_path, "w"))) {
 				fprintf(tf, "%d\n", track_file_init_value);
@@ -709,7 +710,7 @@ process_update_vrrp_track_file_status(const tracked_file_t *tfile, int new_statu
 					    , vrrp->iname, tfile->fname);
 		if (top->weight)
 			vrrp->total_priority -= previous_status;
-		down_instance(vrrp);
+		down_instance(vrrp, VRRP_FAULT_FL_TRACKER);
 	} else if (previous_status == -254) {
 		if (top->weight) {
 			vrrp->total_priority += new_status;
@@ -722,7 +723,7 @@ process_update_vrrp_track_file_status(const tracked_file_t *tfile, int new_statu
 				log_message(LOG_INFO, "(%s) Setting effective priority to %d"
 						    , vrrp->iname, vrrp->effective_priority);
 		}
-		try_up_instance(vrrp, false);
+		try_up_instance(vrrp, false, VRRP_FAULT_FL_TRACKER);
 	} else {
 		vrrp->total_priority += new_status - previous_status;
 		vrrp_set_effective_priority(vrrp);
