@@ -262,7 +262,7 @@ dump_tracking_vrrp(FILE *fp, const void *obj)
 	conf_write(fp, "     %s, weight %d%s%s"
 		     , vrrp->iname, top->weight
 		     , top->weight_multiplier == -1 ? " reverse" : ""
-		     , top->type == TRACK_VRRP_DYNAMIC ? " (dynamic)" : "");
+		     , top->type & TRACK_VRRP_DYNAMIC ? " (dynamic)" : "");
 }
 void
 dump_tracking_vrrp_list(FILE *fp, const list_head_t *l)
@@ -618,6 +618,11 @@ dump_vrrp(FILE *fp, const vrrp_t *vrrp)
 			conf_write(fp, "   Master priority = %d", vrrp->master_priority);
 			if (vrrp->version == VRRP_VERSION_3)
 				conf_write(fp, "   Master advert interval = %u milli-sec", vrrp->master_adver_int / (TIMER_HZ / 1000));
+		} else if (vrrp->state == VRRP_STATE_MAST && vrrp->base_priority == VRRP_PRIO_OWNER) {
+			conf_write(fp, "   Rogue master counter = %u", vrrp->rogue_counter);
+			conf_write(fp, "   Roger timer thread = %p", vrrp->rogue_timer_thread);
+			if (vrrp->rogue_counter || vrrp->rogue_timer_thread)
+				conf_write(fp, "   Roger adver interval = %u ms", vrrp->rogue_adver_int / (TIMER_HZ / 1000));
 		}
 	}
 	if (vrrp->flags) {
@@ -631,10 +636,32 @@ dump_vrrp(FILE *fp, const vrrp_t *vrrp)
 	} else
 		conf_write(fp, "   Flags: none");
 
+	if (vrrp->rlflags)
+		conf_write(fp, "   Rate-limit flags = 0x%x", vrrp->rlflags);
+
 	conf_write(fp, "   Wantstate = %s", get_state_str(vrrp->wantstate));
 	conf_write(fp, "   Number of config faults = %u", vrrp->num_config_faults);
 	if (fp) {
-		conf_write(fp, "   Number of interface and track script faults = %u", vrrp->num_script_if_fault);
+		conf_write(fp, "   Number of tracker faults = %u", vrrp->num_track_fault);
+		if (!vrrp->flags_if_fault)
+			conf_write(fp, "   Flags of interface faults - (none)");
+		else {
+			conf_write(fp, "   Flags of interface faults:");
+			if (__test_bit(VRRP_FAULT_FL_TRACKER, &vrrp->flags_if_fault))
+				conf_write(fp, "     unspecified");
+			if (__test_bit(VRRP_FAULT_FL_INTERFACE_DOWN, &vrrp->flags_if_fault))
+				conf_write(fp, "     i/f down");
+#ifdef _HAVE_VRRP_VMAC_
+			if (__test_bit(VRRP_FAULT_FL_BASE_INTERFACE_DOWN, &vrrp->flags_if_fault))
+				conf_write(fp, "     base i/f down");
+#endif
+			if (__test_bit(VRRP_FAULT_FL_DUPLICATE_VRID, &vrrp->flags_if_fault))
+				conf_write(fp, "     duplicate VRID");
+			if (__test_bit(VRRP_FAULT_FL_NO_SOURCE_IP, &vrrp->flags_if_fault))
+				conf_write(fp, "     no source IP address");
+			if (__test_bit(VRRP_FAULT_FL_CONFIG_ERROR, &vrrp->flags_if_fault))
+				conf_write(fp, "     config error");
+		}
 #ifdef _HAVE_VRRP_VMAC_
 		if (__test_bit(VRRP_FLAG_DUPLICATE_VRID_FAULT, &vrrp->flags))
 			conf_write(fp, "   Duplicate VRID");
@@ -685,7 +712,7 @@ dump_vrrp(FILE *fp, const vrrp_t *vrrp)
 	/* The following should only be specified for VMACs and ipvlans */
 	if (__test_bit(VRRP_VMAC_GROUP, &vrrp->flags))
 		conf_write(fp, "     Interface group %u", vrrp->vmac_group);
-	else if (vrrp->ifp->base_ifp->group)
+	else if (vrrp->ifp && vrrp->ifp->base_ifp->group)
 		conf_write(fp, "     Interface group %u (copied from parent)", vrrp->ifp->base_ifp->group);
 
 	if (vrrp->ifp && vrrp->ifp->is_ours) {
@@ -738,6 +765,8 @@ dump_vrrp(FILE *fp, const vrrp_t *vrrp)
 #endif
 	conf_write(fp, "   Send advert after receive lower priority advert = %s", vrrp->lower_prio_no_advert ? "false" : "true");
 	conf_write(fp, "   Send advert after receive higher priority advert = %s", vrrp->higher_prio_send_advert ? "true" : "false");
+	if (vrrp->base_priority == VRRP_PRIO_OWNER)
+		conf_write(fp, "   Address owner ignores received adverts = %s", vrrp->owner_ignore_adverts ? "true" : "false");
 	conf_write(fp, "   Virtual Router ID = %d", vrrp->vrid);
 	conf_write(fp, "   Priority = %d", vrrp->base_priority);
 	if (fp) {
@@ -988,6 +1017,7 @@ alloc_vrrp(const char *iname)
 #ifdef _WITH_UNICAST_CHKSUM_COMPAT_
 	new->unicast_chksum_compat = CHKSUM_COMPATIBILITY_NONE;
 #endif
+	new->owner_ignore_adverts = PARAMETER_UNSET;
 	if (global_data->v3_checksum_as_v2)
 	        __set_bit(VRRP_FLAG_V3_CHECKSUM_AS_V2, &new->flags);
 	new->smtp_alert = -1;
@@ -1007,6 +1037,7 @@ alloc_vrrp_unicast_peer(const vector_t *strvec)
 	unicast_peer_t *peer;
 	unsigned ttl;
 	unsigned i;
+	unicast_peer_t *existing_peer;
 
 	/* Allocate new unicast peer */
 	PMALLOC(peer);
@@ -1033,7 +1064,7 @@ alloc_vrrp_unicast_peer(const vector_t *strvec)
 
 	for (i = 1; i < vector_size(strvec); i += 2) {
 		if (i + 1 >= vector_size(strvec)) {
-			report_config_error(CONFIG_GENERAL_ERROR, "(%s) %s is missing a value", current_vrrp->iname, strvec_slot(strvec, i));
+			report_config_error(CONFIG_GENERAL_ERROR, "(%s) %s - %s is missing a value", current_vrrp->iname, strvec_slot(strvec, 0), strvec_slot(strvec, i));
 			break;
 		}
 		if (read_unsigned(strvec_slot(strvec, i + 1), &ttl, 0, 255, false)) {
@@ -1042,15 +1073,28 @@ alloc_vrrp_unicast_peer(const vector_t *strvec)
 			else if (!strcmp(strvec_slot(strvec, i), "max_ttl"))
 				peer->max_ttl = ttl;
 			else {
-				report_config_error(CONFIG_GENERAL_ERROR, "(%s) unknown unicast_peer option %s", current_vrrp->iname, strvec_slot(strvec, i));
+				report_config_error(CONFIG_GENERAL_ERROR, "(%s) %s - unknown unicast_peer option %s", current_vrrp->iname, strvec_slot(strvec, 0), strvec_slot(strvec, i));
 				break;
 			}
 			__set_bit(VRRP_FLAG_CHECK_UNICAST_SRC, &current_vrrp->flags);
 		}
 	}
 
+	/* Check this unicast peer is not already configured */
+	list_for_each_entry(existing_peer, &current_vrrp->unicast_peer, e_list) {
+		if ((current_vrrp->family == AF_INET && ((struct sockaddr_in *)&peer->address)->sin_addr.s_addr == ((struct sockaddr_in *)&existing_peer->address)->sin_addr.s_addr) ||
+		    (current_vrrp->family == AF_INET6 &&
+		     !memcmp(&((struct sockaddr_in6 *)&peer->address)->sin6_addr,
+			     &((struct sockaddr_in6 *)&existing_peer->address)->sin6_addr,
+			     sizeof(((struct sockaddr_in6 *)&peer->address)->sin6_addr)))) {
+			report_config_error(CONFIG_GENERAL_ERROR, "(%s) %s - duplicate unicast_peer", current_vrrp->iname, strvec_slot(strvec, 0));
+			FREE(peer);
+			return;
+		}
+	}
+
 	if (peer->min_ttl > peer->max_ttl)
-		report_config_error(CONFIG_GENERAL_ERROR, "(%s) min_ttl %u > max_ttl %u - all packets will be discarded", current_vrrp->iname, peer->min_ttl, peer->max_ttl);
+		report_config_error(CONFIG_GENERAL_ERROR, "(%s) %s - min_ttl %u > max_ttl %u - all packets will be discarded", current_vrrp->iname, strvec_slot(strvec, 0), peer->min_ttl, peer->max_ttl);
 
 	list_add_tail(&peer->e_list, &current_vrrp->unicast_peer);
 }
@@ -1123,6 +1167,43 @@ alloc_vrrp_group_track_bfd(const vector_t *strvec)
 }
 #endif
 
+static bool
+vip_is_duplicate(const ip_address_t *new_ipaddr, const char *vip_str, bool excluded_vip)
+{
+	ip_address_t *vip;
+
+	list_for_each_entry(vip, &current_vrrp->vip, e_list) {
+		/* We can have a VIP and an eVIP the same if they are on
+		 * different interfaces. */
+		if (excluded_vip && new_ipaddr->ifp != vip->ifp)
+			continue;
+
+		if ((IP_FAMILY(new_ipaddr) == AF_INET && new_ipaddr->u.sin.sin_addr.s_addr == vip->u.sin.sin_addr.s_addr) ||
+		    (IP_FAMILY(new_ipaddr) == AF_INET6 && !memcmp(&new_ipaddr->u.sin6_addr, &vip->u.sin6_addr, sizeof(new_ipaddr->u.sin6_addr)))) {
+			report_config_error(CONFIG_GENERAL_ERROR, "(%s): %sVIP duplicates a VIP %s - ignoring",
+					    current_vrrp->iname, excluded_vip ? "Excluded " : "", vip_str);
+			return true;
+		}
+	}
+
+	list_for_each_entry(vip, &current_vrrp->evip, e_list) {
+		if (IP_FAMILY(new_ipaddr) != IP_FAMILY(vip))
+			continue;
+
+		if (new_ipaddr->ifp != vip->ifp)
+			continue;
+
+		if ((IP_FAMILY(new_ipaddr) == AF_INET && new_ipaddr->u.sin.sin_addr.s_addr == vip->u.sin.sin_addr.s_addr) ||
+		    (IP_FAMILY(new_ipaddr) == AF_INET6 && !memcmp(&new_ipaddr->u.sin6_addr, &vip->u.sin6_addr, sizeof(new_ipaddr->u.sin6_addr)))) {
+			report_config_error(CONFIG_GENERAL_ERROR, "(%s): %sVIP duplicates an excluded VIP %s - ignoring",
+					    current_vrrp->iname, excluded_vip ? "Excluded " : "", vip_str);
+			return true;
+		}
+	}
+
+	return false;
+}
+
 void
 alloc_vrrp_vip(const vector_t *strvec)
 {
@@ -1142,6 +1223,12 @@ alloc_vrrp_vip(const vector_t *strvec)
 		return;
 	}
 
+	/* Check we don't already have this address */
+	if (vip_is_duplicate(new_ipaddr, strvec_slot(strvec, 0), false)) {
+		free_ipaddress(new_ipaddr);
+		return;
+	}
+
 	list_add_tail(&new_ipaddr->e_list, &current_vrrp->vip);
 	current_vrrp->vip_cnt++;
 }
@@ -1151,8 +1238,16 @@ alloc_vrrp_evip(const vector_t *strvec)
 {
 	ip_address_t *new_ipaddr;
 
-	if ((new_ipaddr = alloc_ipaddress(strvec, false)))
-		list_add_tail(&new_ipaddr->e_list, &current_vrrp->evip);
+	if (!(new_ipaddr = alloc_ipaddress(strvec, false)))
+		return;
+
+	/* Check we don't already have this address */
+	if (vip_is_duplicate(new_ipaddr, strvec_slot(strvec, 0), true)) {
+		free_ipaddress(new_ipaddr);
+		return;
+	}
+
+	list_add_tail(&new_ipaddr->e_list, &current_vrrp->evip);
 }
 
 void
