@@ -1414,7 +1414,12 @@ interface_down(interface_t *ifp)
 
 		if (route_found) {
 			/* Bring down vrrp instance/sync group */
-			down_instance(vrrp);
+#ifdef _HAVE_VRRP_VMAC_
+			if (__test_bit(VRRP_VMAC_BIT, &vrrp->flags) && VRRP_CONFIGURED_IFP(vrrp) == ifp)
+				down_instance(vrrp, VRRP_FAULT_FL_BASE_INTERFACE_DOWN);
+			else
+#endif
+				down_instance(vrrp, VRRP_FAULT_FL_INTERFACE_DOWN);
 		}
 	}
 
@@ -1451,8 +1456,15 @@ cleanup_lost_interface(interface_t *ifp)
 				if (top->weight) {
 					vrrp->total_priority -= top->weight * top->weight_multiplier;
 					vrrp_set_effective_priority(vrrp);
-				} else
-					down_instance(vrrp);
+				} else {
+#ifdef _HAVE_VRRP_VMAC_
+					if (__test_bit(VRRP_VMAC_BIT, &vrrp->flags) &&
+							VRRP_CONFIGURED_IFP(vrrp) == ifp)
+						down_instance(vrrp, VRRP_FAULT_FL_BASE_INTERFACE_DOWN);
+					else
+#endif
+						down_instance(vrrp, VRRP_FAULT_FL_INTERFACE_DOWN);
+				}
 			}
 			continue;
 		}
@@ -1470,15 +1482,6 @@ cleanup_lost_interface(interface_t *ifp)
 #endif
 		}
 
-		if (vrrp->configured_ifp == ifp &&
-		    vrrp->configured_ifp->base_ifp == vrrp->ifp->base_ifp &&
-		    vrrp->ifp->is_ours) {
-			/* This is a changeable interface that the vrrp instance
-			 * was configured on. Delete the macvlan/ipvlan we created */
-			netlink_link_del_vmac(vrrp);
-// HERE
-		}
-
 		if (vrrp->configured_ifp == ifp &&
 		    vrrp->configured_ifp->base_ifp != vrrp->configured_ifp)
 			del_vrrp_from_interface(vrrp, vrrp->configured_ifp->base_ifp);
@@ -1492,7 +1495,7 @@ cleanup_lost_interface(interface_t *ifp)
 		    vrrp->configured_ifp == ifp &&
 		    __test_bit(VRRP_FLAG_DUPLICATE_VRID_FAULT, &vrrp->flags)) {
 			__clear_bit(VRRP_FLAG_DUPLICATE_VRID_FAULT, &vrrp->flags);
-			vrrp->num_script_if_fault--;
+			__clear_bit(VRRP_FAULT_FL_DUPLICATE_VRID, &vrrp->flags_if_fault);
 		}
 #endif
 
@@ -1509,8 +1512,15 @@ cleanup_lost_interface(interface_t *ifp)
 			}
 		}
 
-		if (IF_ISUP(ifp))
-			down_instance(vrrp);
+		if (IF_ISUP(ifp)) {
+#ifdef _HAVE_VRRP_VMAC_
+			if (__test_bit(VRRP_VMAC_BIT, &vrrp->flags) &&
+					VRRP_CONFIGURED_IFP(vrrp) == ifp)
+				down_instance(vrrp, VRRP_FAULT_FL_BASE_INTERFACE_DOWN);
+			else
+#endif
+				down_instance(vrrp, VRRP_FAULT_FL_INTERFACE_DOWN);
+		}
 	}
 
 	interface_down(ifp);
@@ -1526,6 +1536,22 @@ cleanup_lost_interface(interface_t *ifp)
 	ifp->vrf_master_ifp = NULL;
 	ifp->vrf_master_ifindex = 0;
 #endif
+
+#ifdef _HAVE_VRRP_VMAC_
+	list_for_each_entry(top, &ifp->tracking_vrrp, e_list) {
+		vrrp = top->obj.vrrp;
+		if (!vrrp->ifp)
+			continue;
+
+		if (vrrp->configured_ifp == ifp &&
+		    vrrp->configured_ifp->base_ifp == vrrp->ifp->base_ifp &&
+		    vrrp->ifp->is_ours) {
+			/* This is a changeable interface that the vrrp instance
+			 * was configured on. Delete the macvlan/ipvlan we created */
+			netlink_link_del_vmac(vrrp);
+		}
+	}
+#endif
 }
 
 static void
@@ -1566,13 +1592,22 @@ setup_interface(vrrp_t *vrrp)
 		open_sockpool_socket(vrrp->sockets);
 
 		if (vrrp_initialised) {
-			vrrp->state = vrrp->num_script_if_fault ? VRRP_STATE_FAULT : VRRP_STATE_BACK;
+			vrrp->state = vrrp->flags_if_fault ? VRRP_STATE_FAULT : VRRP_STATE_BACK;
 			vrrp_init_instance_sands(vrrp);
 			vrrp_thread_add_read(vrrp);
 		}
 	}
 
-	return;
+	/* If we have created a VMAC for the vrrp instance, the VMAC has an address
+	 * (IPv6 only) and the vrrp instance does not have an address, add the
+	 * address to the vrrp instance and try to bring it up. */
+	if (vrrp->family == AF_INET6) {
+		if (IN6_IS_ADDR_UNSPECIFIED(&vrrp->saddr) &&
+		    (__test_bit(VRRP_FAULT_FL_NO_SOURCE_IP, &vrrp->flags_if_fault))) {
+			inet_ip6tosockaddr(&vrrp->ifp->sin6_addr, &vrrp->saddr);
+			try_up_instance(vrrp, false, VRRP_FAULT_FL_NO_SOURCE_IP);
+		}
+	}
 }
 
 #ifdef _HAVE_VRRP_VMAC_
@@ -1679,7 +1714,7 @@ update_added_interface(interface_t *ifp)
 				if (IF_BASE_IFP(VRRP_CONFIGURED_IFP(vrrp)) == IF_BASE_IFP(VRRP_CONFIGURED_IFP(vrrp1)) &&
 				    vrrp->family == vrrp1->family &&
 				    vrrp->vrid == vrrp1->vrid) {
-					vrrp->num_script_if_fault++;
+					__set_bit(VRRP_FAULT_FL_DUPLICATE_VRID, &vrrp->flags_if_fault);
 					__set_bit(VRRP_FLAG_DUPLICATE_VRID_FAULT, &vrrp->flags);
 					log_message(LOG_INFO, "VRID conflict between %s and %s IPv%d vrid %d",
 							vrrp->iname, vrrp1->iname, vrrp->family == AF_INET ? 4 : 6, vrrp->vrid);
@@ -1688,13 +1723,20 @@ update_added_interface(interface_t *ifp)
 			}
 		}
 
-		if (vrrp->flags) {
+		if (
+#ifdef _HAVE_VRRP_VMAC_
+		    __test_bit(VRRP_VMAC_BIT, &vrrp->flags)
+#ifdef _HAVE_VRRP_IPVLAN_
+		    || __test_bit(VRRP_IPVLAN_BIT, &vrrp->flags)
+#endif
+#endif
+								) {
 			if (top->type & TRACK_VRRP) {
 				add_vrrp_to_interface(vrrp, ifp->base_ifp, top->weight, top->weight_multiplier == -1, false, TRACK_VRRP_DYNAMIC);
 				if (!IF_ISUP(vrrp->configured_ifp->base_ifp) && !__test_bit(VRRP_FLAG_DONT_TRACK_PRIMARY, &vrrp->flags)) {
 					log_message(LOG_INFO, "(%s) interface %s is down",
 							vrrp->iname, vrrp->configured_ifp->base_ifp->ifname);
-					vrrp->num_script_if_fault++;
+					__set_bit(VRRP_FAULT_FL_BASE_INTERFACE_DOWN, &vrrp->flags_if_fault);
 				}
 			}
 
@@ -1720,7 +1762,10 @@ update_added_interface(interface_t *ifp)
 		/* Reopen any socket on this interface if necessary */
 		if (
 #ifdef _HAVE_VRRP_VMAC_
-		    !vrrp->flags &&
+		    !__test_bit(VRRP_VMAC_BIT, &vrrp->flags) &&
+#ifdef _HAVE_VRRP_IPVLAN_
+		    !__test_bit(VRRP_IPVLAN_BIT, &vrrp->flags) &&
+#endif
 #endif
 		    vrrp->sockets->fd_in == -1)
 			setup_interface(vrrp);
