@@ -32,6 +32,8 @@
 #include <stdio.h>
 #include <inttypes.h>
 
+#include <assert.h>
+
 #include "vrrp_scheduler.h"
 #include "vrrp_track.h"
 #ifdef _HAVE_VRRP_VMAC_
@@ -287,7 +289,7 @@ vrrp_init_state(list_head_t *l)
 			if (vrrp_begin_state != vrrp->state)
 				vrrp->last_transition = timer_now();
 			if (vrrp_begin_state != vrrp->state &&
-			    (vrrp->state != VRRP_STATE_FAULT || vrrp->num_script_if_fault))
+			    (vrrp->state != VRRP_STATE_FAULT || vrrp->flags_if_fault))
 				send_instance_notifies(vrrp);
 			else if (reload && global_data->fifo_write_vrrp_states_on_reload)
 				notify_instance_fifo(vrrp);
@@ -453,7 +455,7 @@ alloc_sock(list_head_t *l, sa_family_t family, int proto, interface_t *ifp,
 #ifdef _HAVE_VRF_
 	   const interface_t *vrf_ifp,
 #endif
-	   const sockaddr_t *mcast_daddr, const sockaddr_t *unicast_src)
+	   sockaddr_t *mcast_daddr, sockaddr_t *unicast_src)
 {
 	sock_t *new;
 
@@ -688,17 +690,38 @@ vrrp_gratuitous_arp_vmac_update_thread(thread_ref_t thread)
 #endif
 
 void
-try_up_instance(vrrp_t *vrrp, bool leaving_init)
+try_up_instance(vrrp_t *vrrp, bool leaving_init, vrrp_fault_fl_t resolved_flag)
 {
 	int wantstate;
 	ip_address_t ip_addr = {0};
 
+	/* We can not use try_up_instance() for several resolution
+	 * at the same time
+	 */
+#ifdef _FAULT_FLAGS_CHECK_
+	if (resolved_flag != VRRP_FAULT_FL_TRACKER && !__test_bit(resolved_flag, &vrrp->flags_if_fault))
+		log_message(LOG_INFO, "(%s) BUG - try_up_instance flag %u not set in 0x%lx, leaving_init %d", vrrp->iname, resolved_flag, vrrp->flags_if_fault, leaving_init);
+
+	if (!__test_bit(VRRP_FAULT_FL_TRACKER, &vrrp->flags_if_fault) != !vrrp->num_track_fault)
+		log_message(LOG_INFO, "(%s) BUG - clear_fault - tracker flag 0x%lx does not match num_track_fault %u", vrrp->iname, vrrp->flags_if_fault, vrrp->num_track_fault);
+#endif
+
 	if (leaving_init) {
-		if (vrrp->num_script_if_fault)
+		if (vrrp->flags_if_fault)
 			return;
+	} else {
+		if (resolved_flag == VRRP_FAULT_FL_TRACKER) {
+			if (!--vrrp->num_track_fault)
+				__clear_bit(VRRP_FAULT_FL_TRACKER, &vrrp->flags_if_fault);
+		} else
+			__clear_bit(resolved_flag, &vrrp->flags_if_fault);
 	}
-	else if (--vrrp->num_script_if_fault || vrrp->num_script_init) {
-		if (!vrrp->num_script_if_fault) {
+
+	if (vrrp->flags_if_fault)
+		return;
+
+	if (vrrp->num_script_init) {
+		if (!vrrp->flags_if_fault) {
 			if (vrrp->sync) {
 				vrrp->sync->num_member_fault--;
 				vrrp->sync->state = VRRP_STATE_INIT;
@@ -827,9 +850,9 @@ vrrp_handle_bfd_event(bfd_event_t * evt)
 			}
 
 			if (!!vbfd->bfd_up == (tbfd->weight_multiplier == 1))
-				try_up_instance(vrrp, false);
+				try_up_instance(vrrp, false, VRRP_FAULT_FL_TRACKER);
 			else
-				down_instance(vrrp);
+				down_instance(vrrp, VRRP_FAULT_FL_TRACKER);
 		}
 
 		break;
@@ -1088,11 +1111,11 @@ vrrp_dispatcher_read(sock_t *sock)
 
 		/* Save non packet data */
 		vrrp->pkt_saddr = src_addr;
-		vrrp->rx_ttl_hop_limit = -1;           /* Default to not received */
+		vrrp->rx_ttl_hl = -1;           /* Default to not received */
 		if (sock->family == AF_INET) {
 			iph = PTR_CAST_CONST(struct iphdr, vrrp_buffer);
 			vrrp->multicast_pkt = IN_MULTICAST(htonl(iph->daddr));
-			vrrp->rx_ttl_hop_limit = iph->ttl;
+			vrrp->rx_ttl_hl = iph->ttl;
 		} else
 			vrrp->multicast_pkt = false;
 		for (cmsg = CMSG_FIRSTHDR(&msghdr); cmsg; cmsg = CMSG_NXTHDR(&msghdr, cmsg)) {
@@ -1102,7 +1125,7 @@ vrrp_dispatcher_read(sock_t *sock)
 
 				if (cmsg->cmsg_type == IPV6_HOPLIMIT &&
 				    cmsg->cmsg_len - sizeof(struct cmsghdr) == sizeof(unsigned int))
-					vrrp->rx_ttl_hop_limit = *PTR_CAST(unsigned int, CMSG_DATA(cmsg));
+					vrrp->rx_ttl_hl = *PTR_CAST(unsigned int, CMSG_DATA(cmsg));
 				else
 				if (cmsg->cmsg_type == IPV6_PKTINFO &&
 				    cmsg->cmsg_len - sizeof(struct cmsghdr) == sizeof(struct in6_pktinfo))
@@ -1220,6 +1243,7 @@ vrrp_script_thread(thread_ref_t thread)
 		/* We don't want the system to be overloaded with scripts that we are executing */
 		log_message(LOG_INFO, "Track script %s is %s, expect idle - skipping run",
 			    vscript->sname, vscript->state == SCRIPT_STATE_RUNNING ? "already running" : "being timed out");
+		return;
 	}
 
 	/* Execute the script in a child process. Parent returns, child doesn't */
