@@ -36,7 +36,7 @@
 	6.6	946753a	15/09/23	ensure CONF_USR_DIR honours configure lib path - uses $(LIBDIR)
 		deb66ac	06/11/23	revert 946753a4
 	6.7	9626923	15/11/23	change using /usr/lib/iproute2 to /usr/share/iproute2
-	6.12	b43f84a	14/10/24	add rt_addrprotos.d subdirectories
+	6.13	b43f84a	14/10/24	add rt_addrprotos.d subdirectories
 
     Debian, Ubuntu, RHEL and openSUSE moved from /etc/iproute2 to /usr/share/iproute2
     Mint, Gentoo and Archlinux currently use /etc/iproute2
@@ -106,6 +106,12 @@ typedef struct _rt_entry {
 	list_head_t	e_list;
 } rt_entry_t;
 
+typedef enum {
+	DIRS_NOT_CHECKED,
+	DIRS_EXIST,
+	DIRS_DONT_EXIST,
+} dir_state_t;
+
 static rt_entry_t const rtntypes[] = {
 	{ RTN_LOCAL, "local", {0}},
 	{ RTN_NAT, "nat", {0}},
@@ -186,6 +192,8 @@ static LIST_HEAD_INITIALIZE(rt_scopes);
 
 static char ret_buf[11];	/* uint32_t in decimal */
 
+static dir_state_t dir_state = DIRS_NOT_CHECKED;
+
 static void
 free_rt_entry(rt_entry_t *rte)
 {
@@ -372,67 +380,83 @@ initialise_list(list_head_t *l, const char *file_name, const rt_entry_t *default
 	if (!list_empty(l))
 		return;
 
-	path = MALLOC(PATH_MAX);
+	if (dir_state == DIRS_NOT_CHECKED) {
 #ifdef IPROUTE_USR_DIR
-	etc_path = MALLOC(PATH_MAX);
+		if (!stat(IPROUTE_USR_DIR, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFDIR)
+			dir_state = DIRS_EXIST;
+		else
+#endif
+		if (!stat(IPROUTE_ETC_DIR, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFDIR)
+			dir_state = DIRS_EXIST;
+		else
+			dir_state = DIRS_DONT_EXIST;
+	}
+
+	if (dir_state == DIRS_EXIST) {
+		path = MALLOC(PATH_MAX);
+#ifdef IPROUTE_USR_DIR
+		etc_path = MALLOC(PATH_MAX);
 #endif
 
-	/* The default location is IPROUTE_USR_DIR, but it is overridden
-	 * if the file exists in IPROUTE_USR_DIR. */
-	snprintf(path, PATH_MAX, "%s/%s", IPROUTE_ETC_DIR, file_name);
-	if (!stat(path, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFREG)
-		read_file(path, l, max);
-#ifdef IPROUTE_USR_DIR
-	else {
-		snprintf(path, PATH_MAX, "%s/%s", IPROUTE_USR_DIR, file_name);
+		/* The default location is IPROUTE_USR_DIR, but it is overridden
+		 * if the file exists in IPROUTE_USR_DIR. */
+		snprintf(path, PATH_MAX, "%s/%s", IPROUTE_ETC_DIR, file_name);
 		if (!stat(path, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFREG)
 			read_file(path, l, max);
-	}
-#endif
-
-	/* iproute2 uses subdirectories for rt_protos, rt_addrprotos, rt_tables
-	 * (and protodown_reasons) as at v6.11.
-	 * To futureproof our code, we will read subdirectories for all files,
-	 * in case iproute2 introduces support for them in the future.
-	 * We need to check all files ending .conf under IPROUTE_USR_DIR and read
-	 * them unless the matching file exists under IPROUTE_ETC_DIR. We then read
-	 * all relevant files under IPROUTE_ETC_DIR. */
 #ifdef IPROUTE_USR_DIR
-	snprintf(path, PATH_MAX, "%s/%s.d", IPROUTE_USR_DIR, file_name);
-	if ((dir = opendir(path))) {
-		while ((ent = readdir(dir))) {
-			if (!wanted_file(path, IPROUTE_USR_DIR, file_name, ent->d_name))
-				continue;
-
-			/* Check if the file exists in IPROUTE_ETC_DIR. We just check if there is a matching
-			 * entry, and don't care what type the entry is */
-			snprintf(etc_path, PATH_MAX, "%s/%s.d/%s", IPROUTE_ETC_DIR, file_name, ent->d_name);
-			if (!stat(etc_path, &statbuf))
-				continue;
-
-			read_file(path, l, max);
+		else {
+			snprintf(path, PATH_MAX, "%s/%s", IPROUTE_USR_DIR, file_name);
+			if (!stat(path, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFREG)
+				read_file(path, l, max);
 		}
-	}
-	closedir(dir);
 #endif
 
-	/* Now read the entries in the IPROUTE_ETC_DIR subdirectory */
-	snprintf(path, PATH_MAX, "%s/%s.d", IPROUTE_ETC_DIR, file_name);
-	if ((dir = opendir(path))) {
-		while ((ent = readdir(dir))) {
-			if (!wanted_file(path, IPROUTE_ETC_DIR, file_name, ent->d_name))
-				continue;
-
-			read_file(path, l, max);
-		}
-	}
-	closedir(dir);
-
-	FREE_PTR(path);
+		/* iproute2 uses subdirectories for rt_protos, rt_addrprotos, rt_tables
+		 * (and protodown_reasons) as at v6.11.
+		 * To futureproof our code, we will read subdirectories for all files,
+		 * in case iproute2 introduces support for them in the future.
+		 * We need to check all files ending .conf under IPROUTE_USR_DIR and read
+		 * them unless the matching file exists under IPROUTE_ETC_DIR. We then read
+		 * all relevant files under IPROUTE_ETC_DIR. */
 #ifdef IPROUTE_USR_DIR
-	FREE_PTR(etc_path);
+		snprintf(path, PATH_MAX, "%s/%s.d", IPROUTE_USR_DIR, file_name);
+		if ((dir = opendir(path))) {
+			while ((ent = readdir(dir))) {
+				if (!wanted_file(path, IPROUTE_USR_DIR, file_name, ent->d_name))
+					continue;
+
+				/* Check if the file exists in IPROUTE_ETC_DIR. We just check if there is a matching
+				 * entry, and don't care what type the entry is */
+				snprintf(etc_path, PATH_MAX, "%s/%s.d/%s", IPROUTE_ETC_DIR, file_name, ent->d_name);
+				if (!stat(etc_path, &statbuf))
+					continue;
+
+				read_file(path, l, max);
+			}
+
+			closedir(dir);
+		}
 #endif
 
+		/* Now read the entries in the IPROUTE_ETC_DIR subdirectory */
+		snprintf(path, PATH_MAX, "%s/%s.d", IPROUTE_ETC_DIR, file_name);
+		if ((dir = opendir(path))) {
+			while ((ent = readdir(dir))) {
+				if (!wanted_file(path, IPROUTE_ETC_DIR, file_name, ent->d_name))
+					continue;
+
+				read_file(path, l, max);
+			}
+
+			closedir(dir);
+		}
+
+		FREE_PTR(path);
+#ifdef IPROUTE_USR_DIR
+		FREE_PTR(etc_path);
+#endif
+	}
+
 	if (default_list)
 		add_default(l, default_list);
 }
@@ -588,7 +612,7 @@ get_rttables_scope(uint32_t id)
 static const char *
 get_rttables_addrproto(uint32_t id)
 {
-	return get_entry(id, &rt_scopes, RT_ADDRPROTOS_FILE, rtscope_default, 255);
+	return get_entry(id, &rt_addrprotos, RT_ADDRPROTOS_FILE, rtaddrproto_default, 255);
 }
 
 static void
@@ -603,7 +627,7 @@ write_addrproto_config(const char *name, uint32_t val)
 	bool file_exists = false;
 	struct stat statbuf;
 
-	fp = popen("ip -V", "re");
+	fp = popen("ip -V 2>&1", "re");
 	res = fgets(buf, sizeof(buf), fp);
 	pclose(fp);
 
@@ -614,15 +638,26 @@ write_addrproto_config(const char *name, uint32_t val)
 	 *     ip utility, iproute2-5.10.0
 	 * or
 	 *     ip utility, iproute2-6.7.0, libbpf 1.2.3
-	 */
+         * or
+         *     BusyBox v1.36.1 (2024-06-10 07:11:47 UTC) multi-call binary
+	 *
+	 * Busybox does not support the iproute2 configuration files.
+         */
+        if (!strstr(buf, "iproute2"))
+                return;
+        if (strstr(buf, "BusyBox"))
+                return;
+
 	if (!(v = strchr(buf, '-')))
 		return;
 
 	v++;
 	if ((e = strchr(v, ',')))
 		*e = '\0';
-	sscanf(v, "%d.%d.%d", &ver_maj, &ver_min, &ver_rel);
-	if (ver_maj >= 7 || (ver_maj == 6 && ver_min >= 12)) {
+	if (sscanf(v, "%d.%d.%d", &ver_maj, &ver_min, &ver_rel) != 3)
+		return;
+
+	if (ver_maj >= 7 || (ver_maj == 6 && ver_min >= 13)) {
 		dir = IPROUTE_ETC_DIR "/" RT_ADDRPROTOS_FILE ".d";
 		path = IPROUTE_ETC_DIR "/" RT_ADDRPROTOS_FILE ".d/keepalived.conf" ;
 	} else if (ver_maj == 6 && ver_min >= 3)
@@ -630,10 +665,28 @@ write_addrproto_config(const char *name, uint32_t val)
 	else
 		return;
 
-	stat(IPROUTE_ETC_DIR, &statbuf);
+	/* If IPROUTE_ETC_DIR doesn't exist, create it */
+	if (stat(IPROUTE_ETC_DIR, &statbuf)) {
+#ifdef IPROUTE_USR_DIR
+		if (stat(IPROUTE_USR_DIR, &statbuf))
+#endif
+		{
+			/* Use sensible defaults for directory permission */
+			statbuf.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+		}
+
+		/* Create directory */
+		if (mkdir(IPROUTE_ETC_DIR, statbuf.st_mode & ~S_IFMT)) {
+			log_message(LOG_INFO, "Unable to create directory " IPROUTE_ETC_DIR " for rt_addrproto keepalived");
+			return;
+		}
+	}
+
 	if (dir) {
 		if (!mkdir(dir, statbuf.st_mode & ~S_IFMT))	// This may fail if the directory already exists
 			chmod(dir, statbuf.st_mode & ~S_IFMT);
+		else if (errno == EEXIST)
+			file_exists = !stat(path, &statbuf);
 	} else {
 		/* Check if rt_addrprotos file exists */
 		file_exists = !stat(path, &statbuf);
@@ -685,7 +738,8 @@ create_rttables_addrproto(const char *name, uint8_t *id)
 	list_add_tail(&rte->e_list, &rt_addrprotos);
 
 	/* Save the entry so iproute can use it */
-	write_addrproto_config(name, *id);
+	if (dir_state != DIRS_DONT_EXIST)
+		write_addrproto_config(name, *id);
 
 	return true;
 }
